@@ -28,6 +28,8 @@
 
 #include <config.h>
 
+#include <stdio.h>
+
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gst/gst.h>
@@ -49,7 +51,6 @@ typedef enum {
 
 typedef GstElement *(*RBAddTaggerElem) (GstElement *pipeline, GstPad *srcpad, GstTagList *tags);
 
-static gboolean check_gst_plugin_version (const char *plugin, const char *element, gint major, gint minor, gint micro);
 G_DEFINE_TYPE(RBMetaData, rb_metadata, G_TYPE_OBJECT)
 
 struct RBMetaDataPrivate
@@ -103,7 +104,7 @@ run_typefind (RBMetaData *md, const char *uri)
 {
 	GstElement *src;
 
-	src = gst_element_make_from_uri (GST_URI_SRC, uri, NULL);
+	src = gst_element_make_from_uri (GST_URI_SRC, uri, NULL, NULL);
 	if (src != NULL) {
 		GstElement *pipeline = gst_pipeline_new (NULL);
 		GstElement *sink = gst_element_factory_make ("fakesink", NULL);
@@ -148,8 +149,11 @@ run_typefind (RBMetaData *md, const char *uri)
 				default:
 					break;
 				}
+
+				gst_message_unref (message);
 			}
 
+			g_object_unref (bus);
 			gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL);
 		}
 
@@ -167,7 +171,10 @@ rb_metadata_load (RBMetaData *md, const char *uri, GError **error)
 
 	rb_metadata_reset (md);
 
-	discoverer = gst_discoverer_new (30 * GST_SECOND, NULL);
+	discoverer = gst_discoverer_new (30 * GST_SECOND, error);
+	if (*error != NULL)
+		return;
+
 	md->priv->info = gst_discoverer_discover_uri (discoverer, g_strdup (uri), error);
 	g_object_unref (discoverer);
 
@@ -332,31 +339,6 @@ rb_metadata_get_saveable_types (RBMetaData *md)
 	return types;
 }
 
-static gboolean
-check_gst_plugin_version (const char *plugin, const char *element, int major, int minor, int micro)
-{
-	const char *version;
-	GstPlugin *p;
-	guint i;
-	guint count;
-
-	if (gst_default_registry_check_feature_version (element, major, minor, micro + 1))
-		return TRUE;
-
-	if (!gst_default_registry_check_feature_version (element, major, minor, micro))
-		return FALSE;
-
-	p = gst_default_registry_find_plugin (plugin);
-	if (p == NULL)
-		return FALSE;
-
-	version = gst_plugin_get_version (p);
-
-	/* check if it's not a release */	/* what */
-	count = sscanf (version, "%u.%u.%u.%u", &i, &i, &i, &i);
-	return (count > 3);
-}
-
 static gboolean
 link_named_pad (GstPad *srcpad, GstElement *element, const char *sinkpadname)
 {
@@ -411,10 +393,7 @@ mp3_tagger (GstElement *pipeline, GstPad *srcpad, GstTagList *tags)
 	GstElement *mux = NULL;
 
 	/* try id3mux first, since it's more supported and writes id3v2.3 tags rather than v2.4.  */
-	if (check_gst_plugin_version ("id3mux", "id3mux", 0, 10, 12)) {
-		mux = gst_element_factory_make ("id3mux", NULL);
-	}
-
+	mux = gst_element_factory_make ("id3mux", NULL);
 	if (mux == NULL)
 		mux = gst_element_factory_make ("id3v2mux", NULL);
 
@@ -488,7 +467,7 @@ mp4_tagger (GstElement *pipeline, GstPad *srcpad, GstTagList *tags)
 		return NULL;
 
 	gst_bin_add (GST_BIN (pipeline), mux);
-	if (!link_named_pad (srcpad, mux, "audio_%d"))
+	if (!link_named_pad (srcpad, mux, "audio_%u"))
 		goto error;
 
 	gst_element_set_state (mux, GST_STATE_PAUSED);
@@ -503,7 +482,7 @@ error:
 }
 
 static void
-metadata_save_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean last, RBMetaData *md)
+metadata_save_pad_added_cb (GstElement *decodebin, GstPad *pad, RBMetaData *md)
 {
 	RBAddTaggerElem add_tagger_func = NULL;
 	GstElement *retag_end;
@@ -527,7 +506,7 @@ metadata_save_new_decoded_pad_cb (GstElement *decodebin, GstPad *pad, gboolean l
 	}
 
 	/* find a tagger function that accepts the caps */
-	caps = gst_pad_get_caps (pad);
+	caps = gst_pad_query_caps (pad, NULL);
 	caps_str = gst_caps_to_string (caps);
 	rb_debug ("finding tagger for src caps %s", caps_str);
 	g_free (caps_str);
@@ -612,7 +591,7 @@ metadata_save_autoplug_select_cb (GstElement *decodebin, GstPad *pad, GstCaps *c
 	}
 
 	src_caps = gst_caps_new_any ();
-	is_any = gst_element_factory_can_src_caps (factory, src_caps);
+	is_any = gst_element_factory_can_src_all_caps (factory, src_caps);	/* or _any_caps? */
 	gst_caps_unref (src_caps);
 	if (is_any) {
 		/* this is something like id3demux (though that will match the
@@ -621,7 +600,7 @@ metadata_save_autoplug_select_cb (GstElement *decodebin, GstPad *pad, GstCaps *c
 		return GST_AUTOPLUG_SELECT_TRY;
 	}
 
-	src_caps = gst_caps_from_string ("audio/x-raw-int; audio/x-raw-float; video/x-raw-yuv; video/x-raw-rgb; video/x-raw-gray");
+	src_caps = gst_caps_from_string ("audio/x-raw; video/x-raw");
 	is_raw = factory_src_caps_intersect (factory, src_caps);
 	gst_caps_unref (src_caps);
 
@@ -684,7 +663,7 @@ rb_metadata_save (RBMetaData *md, const char *uri, GError **error)
 	md->priv->pipeline = pipeline;
 	md->priv->sink_linked = FALSE;
 
-	urisrc = gst_element_make_from_uri (GST_URI_SRC, uri, "urisrc");
+	urisrc = gst_element_make_from_uri (GST_URI_SRC, uri, "urisrc", NULL);
 	if (urisrc == NULL) {
 		g_set_error (error,
 			     RB_METADATA_ERROR,
@@ -694,12 +673,12 @@ rb_metadata_save (RBMetaData *md, const char *uri, GError **error)
 		goto out;
 	}
 
-	decodebin = gst_element_factory_make ("decodebin2", "decoder");
+	decodebin = gst_element_factory_make ("decodebin", "decoder");
 	if (decodebin == NULL) {
 		g_set_error (error,
 			     RB_METADATA_ERROR,
 			     RB_METADATA_ERROR_MISSING_PLUGIN,
-			     _("Failed to create the 'decodebin2' element; check your GStreamer installation"));
+			     _("Failed to create the 'decodebin' element; check your GStreamer installation"));
 		goto out;
 	}
 
@@ -717,8 +696,8 @@ rb_metadata_save (RBMetaData *md, const char *uri, GError **error)
 	gst_element_link (urisrc, decodebin);
 
 	g_signal_connect_object (decodebin,
-				 "new-decoded-pad",
-				 G_CALLBACK (metadata_save_new_decoded_pad_cb),
+				 "pad-added",
+				 G_CALLBACK (metadata_save_pad_added_cb),
 				 md, 0);
 	g_signal_connect_object (decodebin,
 				 "autoplug-select",
@@ -844,6 +823,9 @@ rb_metadata_get (RBMetaData *md, RBMetaDataField field, GValue *ret)
 	GValue gstvalue = {0, };
 	GstClockTime duration;
 
+	if (md->priv->info == NULL)
+		return FALSE;
+
 	/* special cases: mostly duration */
 	switch (field) {
 	case RB_METADATA_FIELD_DURATION:
@@ -945,7 +927,7 @@ rb_metadata_set (RBMetaData *md, RBMetaDataField field, const GValue *val)
 			rb_debug ("Setting %s",tag);
 
 			if (md->priv->tags == NULL) {
-				md->priv->tags = gst_tag_list_new ();
+				md->priv->tags = gst_tag_list_new_empty ();
 			}
 
 			gst_tag_list_add_values (md->priv->tags,
@@ -991,8 +973,7 @@ rb_metadata_init (RBMetaData *md)
 			g_hash_table_insert (md->priv->taggers, "audio/x-flac", flac_tagger);
 		}
 
-		/* id3mux > 0.10.12 is the new one, not the broken old one. */
-		if (gst_element_factory_find ("id3v2mux") || check_gst_plugin_version ("id3mux", "id3mux", 0, 10, 12)) {
+		if (gst_element_factory_find ("id3v2mux") || gst_element_factory_find ("id3mux")) {
 			rb_debug ("id3 tagging available");
 			g_hash_table_insert (md->priv->taggers, "audio/mpeg", mp3_tagger);
 		}
@@ -1014,7 +995,6 @@ rb_metadata_class_init (RBMetaDataClass *klass)
 	object_class->finalize = rb_metadata_finalize;
 
 	g_type_class_add_private (klass, sizeof (RBMetaDataPrivate));
-	rb_metadata_gst_register_transforms ();
 }
 
 RBMetaData *
