@@ -88,6 +88,7 @@ struct RBPropertyViewPrivate
 
 	gboolean draggable;
 	gboolean handling_row_deletion;
+	guint reset_selection_id;
 };
 
 #define RB_PROPERTY_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_PROPERTY_VIEW, RBPropertyViewPrivate))
@@ -368,6 +369,11 @@ rb_property_view_dispose (GObject *object)
 
 	view = RB_PROPERTY_VIEW (object);
 
+	if (view->priv->reset_selection_id != 0) {
+		g_source_remove (view->priv->reset_selection_id);
+		view->priv->reset_selection_id = 0;
+	}
+
 	rb_property_view_set_model_internal (view, NULL);
 
 	G_OBJECT_CLASS (rb_property_view_parent_class)->dispose (object);
@@ -566,6 +572,27 @@ rb_property_view_pre_row_deleted_cb (RhythmDBPropertyModel *model,
 	rb_debug ("pre row deleted");
 }
 
+static gboolean
+reset_selection_cb (RBPropertyView *view)
+{
+	GtkTreeIter first_iter;
+	if ((gtk_tree_selection_count_selected_rows (view->priv->selection) == 0) &&
+	    gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &first_iter)) {
+		rb_debug ("no rows selected, signalling reset");
+		g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
+						 G_CALLBACK (rb_property_view_selection_changed_cb),
+						 view);
+		gtk_tree_selection_select_iter (view->priv->selection, &first_iter);
+		g_signal_emit (G_OBJECT (view), rb_property_view_signals[SELECTION_RESET], 0);
+		g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
+						   G_CALLBACK (rb_property_view_selection_changed_cb),
+						   view);
+	}
+
+	view->priv->reset_selection_id = 0;
+	return FALSE;
+}
+
 static void
 rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 				      GtkTreePath *path,
@@ -574,17 +601,15 @@ rb_property_view_post_row_deleted_cb (GtkTreeModel *model,
 	view->priv->handling_row_deletion = FALSE;
 	rb_debug ("post row deleted");
 	if (gtk_tree_selection_count_selected_rows (view->priv->selection) == 0) {
-		GtkTreeIter first_iter;
-		rb_debug ("no rows selected, signalling reset");
-		if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (view->priv->prop_model), &first_iter)) {
-			g_signal_handlers_block_by_func (G_OBJECT (view->priv->selection),
-							 G_CALLBACK (rb_property_view_selection_changed_cb),
-							 view);
-			gtk_tree_selection_select_iter (view->priv->selection, &first_iter);
-			g_signal_emit (G_OBJECT (view), rb_property_view_signals[SELECTION_RESET], 0);
-			g_signal_handlers_unblock_by_func (G_OBJECT (view->priv->selection),
-							   G_CALLBACK (rb_property_view_selection_changed_cb),
-							   view);
+		if (view->priv->reset_selection_id == 0) {
+			/* we could be in the middle of renaming a property.
+			 * we've seen it being removed, but we haven't seen the
+			 * new name being added yet, which means the property model
+			 * is out of sync with the query model, so we can't
+			 * reset the selection here.  wait until we've finished
+			 * processing the change.
+			 */
+			view->priv->reset_selection_id = g_idle_add ((GSourceFunc) reset_selection_cb, view);
 		}
 	}
 }
@@ -669,12 +694,8 @@ rb_property_view_constructed (GObject *object)
 
 	view = RB_PROPERTY_VIEW (object);
 
-	view->priv->prop_model = rhythmdb_property_model_new (view->priv->db, view->priv->propid);
 	view->priv->treeview = GTK_WIDGET (gtk_tree_view_new_with_model (GTK_TREE_MODEL (view->priv->prop_model)));
 
-	if (view->priv->draggable)
-		rhythmdb_property_model_enable_drag (view->priv->prop_model,
-						     GTK_TREE_VIEW (view->priv->treeview));
 
 	g_signal_connect_object (G_OBJECT (view->priv->treeview),
 			         "row_activated",
@@ -702,6 +723,11 @@ rb_property_view_constructed (GObject *object)
 
 	gtk_container_add (GTK_CONTAINER (view), view->priv->treeview);
 
+	rb_property_view_set_model_internal (view, rhythmdb_property_model_new (view->priv->db, view->priv->propid));
+	if (view->priv->draggable)
+		rhythmdb_property_model_enable_drag (view->priv->prop_model,
+						     GTK_TREE_VIEW (view->priv->treeview));
+
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view->priv->treeview), TRUE);
 	gtk_tree_selection_set_mode (view->priv->selection, GTK_SELECTION_SINGLE);
 
@@ -880,11 +906,6 @@ rb_property_view_selection_changed_cb (GtkTreeSelection *selection,
 					    RHYTHMDB_PROPERTY_MODEL_COLUMN_PRIORITY, &is_all, -1);
 			g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTY_SELECTED], 0,
 				       is_all ? NULL : selected_prop);
-		} else {
-			if (gtk_tree_model_get_iter_first (model, &iter))
-				gtk_tree_selection_select_iter (selection, &iter);
-			g_signal_emit (G_OBJECT (view), rb_property_view_signals[PROPERTY_SELECTED], 0,
-				       NULL);
 		}
 	}
 
